@@ -1,7 +1,7 @@
 use crate::util::{AsUsize};
 use super::{
     buffer, code,
-    Patch, Label, Counter, Pool, RESULT,
+    Patch, Label, Pool, RESULT,
     Offset, Shift, Unsigned,
     Register, RSP, Condition, MemOp, ShiftOp, AddOp, LogicOp,
     Assembler, CALLEE_SAVES, CALLER_SAVES, ARGUMENTS, RESULTS,
@@ -222,11 +222,6 @@ impl<B: Buffer> Lowerer<B> {
         (POOL, (self.pool.index_of_global(global) * 8) as u64)
     }
 
-    /** Returns the base and offset of `counter`. */
-    fn counter_address(&self, counter: Counter) -> (Register, u64) {
-        (POOL, (self.pool.index_of_counter(counter) * 8) as u64)
-    }
-
     /** Returns the base and offset of `slot` in the stack-allocated data. */
     fn slot_address(&self, slot: Slot) -> (Register, u64) {
         assert!(slot.0 < self.slots_used);
@@ -507,12 +502,6 @@ impl<B: Buffer> super::Lower for Lowerer<B> {
             },
         };
     }
-
-    fn count(&mut self, counter: Counter) {
-        self.mem(LDR, TEMP0, self.counter_address(counter), TEMP1);
-        self.const_add(ADD, P64, TEMP0, TEMP0, 1, TEMP1);
-        self.mem(STR, TEMP0, self.counter_address(counter), TEMP1);
-    }
 }
 
 //-----------------------------------------------------------------------------
