@@ -1,7 +1,7 @@
 use crate::util::{AsUsize};
 use super::{
     buffer, code,
-    Patch, Label, Counter, Word, Pool, RESULT,
+    Patch, Label, Word, Pool, RESULT,
     Assembler, Register, BinaryOp, ShiftOp, Condition, Width,
     CALLEE_SAVES, ARGUMENTS, RESULTS,
 };
@@ -197,11 +197,6 @@ impl<B: Buffer> Lowerer<B> {
         (POOL, (self.pool.index_of_global(global) * 8) as i32)
     }
 
-    /** Returns the base and offset of `counter`. */
-    fn counter_address(&self, counter: Counter) -> (Register, i32) {
-        (POOL, (self.pool.index_of_counter(counter) * 8) as i32)
-    }
-
     /** Returns the base and offset of `slot` in the stack-allocated data. */
     fn slot_address(&self, slot: Slot) -> (Register, i32) {
         assert!(slot.0 < self.slots_used);
@@ -498,14 +493,22 @@ impl<B: Buffer> Lowerer<B> {
             },
             code::BinaryOp::Max => {
                 self.compare_binary(prec, dest, src1, src2, |l, dest, src1| {
-                    l.move_(dest, src1);
-                    l.value_move_if(Condition::L, prec, dest, src2);
+                    if Value::Register(dest) == src2.into() {
+                        l.value_move_if(Condition::GE, prec, dest, src1);
+                    } else {
+                        l.move_(dest, src1);
+                        l.value_move_if(Condition::L, prec, dest, src2);
+                    }
                 });
             },
             code::BinaryOp::Min => {
                 self.compare_binary(prec, dest, src1, src2, |l, dest, src1| {
-                    l.move_(dest, src1);
-                    l.value_move_if(Condition::G, prec, dest, src2);
+                    if Value::Register(dest) == src2.into() {
+                        l.value_move_if(Condition::LE, prec, dest, src1);
+                    } else {
+                        l.move_(dest, src1);
+                        l.value_move_if(Condition::G, prec, dest, src2);
+                    }
                 });
             },
         };
@@ -676,13 +679,6 @@ impl<B: Buffer> super::Lower for Lowerer<B> {
             },
         };
     }
-
-    fn count(&mut self, counter: Counter) {
-        // This could be a single instruction.
-        self.a.load(P64, TEMP, self.counter_address(counter));
-        self.a.const_op(BinaryOp::Add, P64, TEMP, 1);
-        self.a.store(P64, self.counter_address(counter), TEMP);
-    }
 }
 
 //-----------------------------------------------------------------------------
